import logging
import sys

web_runner_logger = logging.getLogger("WEBRunner")
web_runner_logger.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s')
# Stream handler
stream_handler = logging.StreamHandler(stream=sys.stderr)
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.WARNING)
web_runner_logger.addHandler(stream_handler)
# File handler
file_handler = logging.FileHandler(filename="WEBRunner.log", mode="w")
file_handler.setFormatter(formatter)
web_runner_logger.addHandler(file_handler)



import typing

from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement

from je_web_runner.utils.exception.exceptions import WebRunnerAssertException


def _make_webdriver_check_dict(webdriver_to_check: WebDriver) -> dict:
    """
    use to check webdriver current info
    :param webdriver_to_check: what webdriver we want to check
    :return: webdriver detail dict
    """
    webdriver_detail_dict = dict()
    webdriver_detail_dict.update(
        {
            "mobile": webdriver_to_check.mobile,
            "name": webdriver_to_check.name,
            "title": webdriver_to_check.title,
            "current_url": webdriver_to_check.current_url,
            "page_source": webdriver_to_check.page_source,
            "current_window_handle": webdriver_to_check.current_window_handle,
            "window_handles": webdriver_to_check.window_handles,
            "switch_to": webdriver_to_check.switch_to,
            "timeouts": webdriver_to_check.timeouts,
            "capabilities": webdriver_to_check.capabilities,
            "file_detector": webdriver_to_check.file_detector,
            "application_cache": webdriver_to_check.application_cache,
            "virtual_authenticator_id": webdriver_to_check.virtual_authenticator_id
        }
    )
    return webdriver_detail_dict


def _make_web_element_check_dict(web_element_to_check: WebElement) -> dict:
    """
    use to check web element current info
    :param web_element_to_check: what web element we want to check
    :return: web element detail dict
    """
    web_element_detail_dict = dict()
    web_element_detail_dict.update(
        {
            "tag_name": web_element_to_check.tag_name,
            "text": web_element_to_check.text,
            "location_once_scrolled_into_view": web_element_to_check.location_once_scrolled_into_view,
            "size": web_element_to_check.size,
            "location": web_element_to_check.location,
            "parent": web_element_to_check.parent,
            "id": web_element_to_check.id,
        }
    )
    return web_element_detail_dict


def check_value(element_name: str, element_value: typing.Any, result_check_dict: dict) -> None:
    """
    use to check state
    :param element_name: the name of element we want to check
    :param element_value: what value element should be
    :param result_check_dict: the dict include data name and value to check check_dict is valid or not
    :return: None
    """
    if result_check_dict.get(element_name) != element_value:
        raise WebRunnerAssertException(
            "value should be {right_value} but value was {wrong_value}".format(
                right_value=element_value, wrong_value=result_check_dict.get(element_name)
            )
        )


def check_values(check_dict: dict, result_check_dict: dict) -> None:
    """
    :param check_dict: dict include data name and value to check
    :param result_check_dict: the dict include data name and value to check check_dict is valid or not
    :return: None
    """
    for key, value in result_check_dict.items():
        if check_dict.get(key) != value:
            raise WebRunnerAssertException(
                "value should be {right_value} but value was {wrong_value}".format(
                    right_value=value, wrong_value=check_dict.get(key)
                )
            )


def check_webdriver_value(element_name: str, element_value: typing.Any, webdriver_to_check: WebDriver) -> None:
    """
    :param element_name: the name of element we want to check
    :param element_value: what value element should be
    :param webdriver_to_check: the dict include data name and value to check result_dict is valid or not
    :return: None
    """
    check_dict = _make_webdriver_check_dict(webdriver_to_check)
    check_value(element_name, element_value, check_dict)


def check_webdriver_details(webdriver_to_check: WebDriver, result_check_dict: dict) -> None:
    """
    :param webdriver_to_check: what webdriver we want to check
    :param result_check_dict: the dict include data name and value to check result_dict is valid or not
    :return: None
    """
    check_dict = _make_webdriver_check_dict(webdriver_to_check)
    check_values(check_dict, result_check_dict)


def check_web_element_value(element_name: str, element_value: typing.Any, web_element_to_check: WebElement) -> None:
    """
    :param element_name: the name of element we want to check
    :param element_value: what value element should be
    :param web_element_to_check: the dict include data name and value to check result_dict is valid or not
    :return: None
    """
    check_dict = _make_web_element_check_dict(web_element_to_check)
    check_value(element_name, element_value, check_dict)


def check_web_element_details(web_element_to_check: WebElement, result_check_dict: dict) -> None:
    """
    :param web_element_to_check: what web element we want to check
    :param result_check_dict: the dict include data name and value to check result_dict is valid or not
    :return: None
    """
    check_dict = _make_web_element_check_dict(web_element_to_check)
    check_values(check_dict, result_check_dict)



import datetime
from typing import Union

from je_web_runner.utils.logging.loggin_instance import web_runner_logger


class TestRecord(object):

    def __init__(self, init_record: bool = False):
        self.test_record_list: list = list()
        self.init_record: bool = init_record

    def clean_record(self) -> None:
        self.test_record_list = list()

    def set_record_enable(self, set_enable: bool = True):
        web_runner_logger.info(f"set_record_enable, set_enable: {set_enable}")
        self.init_record = set_enable


test_record_instance = TestRecord()


def record_action_to_list(function_name: str, local_param: Union[vars, None],
                          program_exception: Union[Exception, None] = None):
    """
    :param function_name: what function call this method
    :param local_param: what param used
    :param program_exception: what exception happened
    :return: None
    """
    if not test_record_instance.init_record:
        pass
    else:
        test_record_instance.test_record_list.append({
            "function_name": function_name,
            "local_param": local_param,
            "time": str(datetime.datetime.now()),
            "program_exception": repr(program_exception)}
        )



selenium_wrapper_web_driver_not_found_error = "Web Driver not found"

selenium_wrapper_set_options_error = "only accept dict type"
selenium_wrapper_set_argument_error = "only accept str type"

executor_data_error = "executor receive wrong data"
executor_list_error = "executor receive wrong data list is none or wrong type"
# json
cant_find_json_error = "can't find json"
cant_save_json_error = "can't save json"
cant_reformat_json_error = "can't reformat json is type right?"
cant_find_element_in_json_error = "can't find element in json"
json_type_error = "json type error"
wrong_json_data_error = "can't parser json"
cant_generate_json_report: str = "can't generate json report"

html_generate_no_data_tag = "can't make html report, data is null?"
# add command
add_command_exception_tag = "command value type should be as method or function"
# argparse
argparse_get_wrong_data = "argparse receive wrong data"
# XML
cant_read_xml_error: str = "can't read xml"
xml_type_error: str = "xml type error"
# Callback executor
get_bad_trigger_method: str = "get bad trigger method, only accept kwargs and args"
get_bad_trigger_function: str = "get bad trigger function only accept function in event_dict"



import sys
from threading import Lock
from xml.dom.minidom import parseString

from je_web_runner.utils.generate_report.generate_json_report import generate_json
from je_web_runner.utils.logging.loggin_instance import web_runner_logger
from je_web_runner.utils.xml.change_xml_structure.change_xml_structure import dict_to_elements_tree


def generate_xml():
    """
    :return: success test dict and failure test dict
    """
    web_runner_logger.info(f"generate_xml")
    success_dict, failure_dict = generate_json()
    success_dict = dict({"xml_data": success_dict})
    failure_dict = dict({"xml_data": failure_dict})
    success_json_to_xml = dict_to_elements_tree(success_dict)
    failure_json_to_xml = dict_to_elements_tree(failure_dict)
    return success_json_to_xml, failure_json_to_xml


def generate_xml_report(xml_file_name: str = "default_name"):
    """
    :param xml_file_name: save xml use xml_file_name
    """
    web_runner_logger.info(f"generate_xml_report, xml_file_name: {xml_file_name}")
    success_xml, failure_xml = generate_xml()
    success_xml = parseString(success_xml)
    failure_xml = parseString(failure_xml)
    success_xml = success_xml.toprettyxml()
    failure_xml = failure_xml.toprettyxml()
    lock = Lock()
    try:
        lock.acquire()
        with open(xml_file_name + "_failure.xml", "w+") as file_to_write:
            file_to_write.write(failure_xml)
    except Exception as error:
        web_runner_logger.error(f"generate_xml_report, xml_file_name: {xml_file_name}, failed: {repr(error)}")
    finally:
        lock.release()
    try:
        lock.acquire()
        with open(xml_file_name + "_success.xml", "w+") as file_to_write:
            file_to_write.write(success_xml)
            pass
    except Exception as error:
        web_runner_logger.error(f"generate_xml_report, xml_file_name: {xml_file_name}, failed: {repr(error)}")
    finally:
        lock.release()




from sys import stderr

from selenium.common.exceptions import WebDriverException
from selenium.webdriver.remote.webdriver import WebDriver

from je_web_runner.je_web_runner.element.web_element_wrapper import web_element_wrapper
from je_web_runner.je_web_runner.webdriver.webdriver_wrapper import webdriver_wrapper
from je_web_runner.utils.exception.exception_tags import selenium_wrapper_web_driver_not_found_error
from je_web_runner.utils.exception.exceptions import WebRunnerWebDriverIsNoneException
from je_web_runner.utils.logging.loggin_instance import web_runner_logger
from je_web_runner.utils.test_object.test_object_record.test_object_record_class import test_object_record
from je_web_runner.utils.test_record.test_record_class import record_action_to_list


class WebdriverManager(object):

    def __init__(self, **kwargs):
        self._current_webdriver_list = list()
        self.webdriver_wrapper = webdriver_wrapper
        self.webdriver_element = web_element_wrapper
        self.current_webdriver: [WebDriver, None] = None

    def new_driver(self, webdriver_name: str, **kwargs) -> None:
        """
        use to create new webdriver instance
        :param webdriver_name: which webdriver we want to use [chrome, chromium, firefox, edge, ie]
        :param kwargs: webdriver download manager param
        :return: None
        """
        web_runner_logger.info(f"WebdriverManager new_driver, webdriver_name: {webdriver_name}, params: {kwargs}")
        param = locals()
        try:
            self.current_webdriver = webdriver_wrapper.set_driver(webdriver_name, **kwargs)
            self._current_webdriver_list.append(self.current_webdriver)
            record_action_to_list("web runner manager new_driver", param, None)
        except Exception as error:
            web_runner_logger.error(
                f"WebdriverManager new_driver, webdriver_name: {webdriver_name}, params: {kwargs}, failed: {repr(error)}"
            )
            record_action_to_list("web runner manager new_driver", param, error)
            self.quit()

    def change_webdriver(self, index_of_webdriver: int) -> None:
        """
        change to target webdriver
        :param index_of_webdriver: change current webdriver to choose index webdriver
        :return: None
        """
        web_runner_logger.info(f"WebdriverManager change_webdriver, index_of_webdriver: {index_of_webdriver}")
        param = locals()
        try:
            self.current_webdriver = self._current_webdriver_list[index_of_webdriver]
            self.webdriver_wrapper.current_webdriver = self.current_webdriver
            record_action_to_list("web runner manager change_webdriver", param, None)
        except Exception as error:
            web_runner_logger.error(
                f"WebdriverManager change_webdriver, index_of_webdriver: {index_of_webdriver}, failed: {repr(error)}")
            record_action_to_list("web runner manager change_webdriver", param, error)

    def close_current_webdriver(self) -> None:
        """
        close current webdriver
        :return: None
        """
        web_runner_logger.info(f"WebdriverManager close_current_webdriver")
        try:
            self._current_webdriver_list.remove(self.current_webdriver)
            self.current_webdriver.close()
            record_action_to_list("web runner manager close_current_webdriver", None, None)
        except Exception as error:
            web_runner_logger.error(f"WebdriverManager close_current_webdriver, failed: {repr(error)}")
            record_action_to_list("web runner manager close_current_webdriver", None, error)

    def close_choose_webdriver(self, webdriver_index: int) -> None:
        """
        close choose webdriver
        :param webdriver_index: close choose webdriver on current webdriver list
        :return: None
        """
        web_runner_logger.info(f"WebdriverManager close_choose_webdriver")
        param = locals()
        try:
            self.current_webdriver = self._current_webdriver_list[webdriver_index]
            self.current_webdriver.close()
            self._current_webdriver_list.remove(self._current_webdriver_list[webdriver_index])
            record_action_to_list("web runner manager close_choose_webdriver", param, None)
        except Exception as error:
            web_runner_logger.info(f"WebdriverManager close_choose_webdriver, failed: {repr(error)}")
            record_action_to_list("web runner manager close_choose_webdriver", param, error)

    def quit(self) -> None:
        """
        close and quit all webdriver instance
        :return: None
        """
        web_runner_logger.info(f"WebdriverManager quit")
        try:
            if self._current_webdriver_list is None:
                raise WebRunnerWebDriverIsNoneException(selenium_wrapper_web_driver_not_found_error)
            test_object_record.clean_record()
            for webdriver in self._current_webdriver_list:
                webdriver.quit()
            self._current_webdriver_list = list()
            record_action_to_list("web runner manager quit", None, None)
        except Exception as error:
            web_runner_logger.info(f"WebdriverManager quit, failed: {repr(error)}")
            record_action_to_list("web runner manager quit", None, error)
            raise WebDriverException


def get_webdriver_manager(webdriver_name: str, **kwargs) -> WebdriverManager:
    """
    use to get webdriver instance
    :param webdriver_name: which webdriver we want to use [chrome, chromium, firefox, edge, ie]
    :param kwargs: webdriver download manager param
    :return: Webdriver manager instance
    """
    web_runner_logger.info(f"get_webdriver_manager, webdriver_name: {webdriver_name}, params: {kwargs}")
    web_runner.new_driver(webdriver_name, **kwargs)
    return web_runner


web_runner = WebdriverManager()
